@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2014, 2015 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013, 2014, 2015, 2016, 2017 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@ using namespace WhiskerMenu;
 
 WhiskerMenu::Window::Window() :
 	m_window(NULL),
+	m_sidebar_size_group(NULL),
 	m_layout_left(true),
 	m_layout_bottom(true),
 	m_layout_search_alternate(false),
@@ -73,13 +74,10 @@ WhiskerMenu::Window::Window() :
 	g_signal_connect_slot(m_window, "configure-event", &Window::on_configure_event, this);
 	g_signal_connect(m_window, "delete_event", G_CALLBACK(gtk_widget_hide_on_delete), NULL);
 
-	m_window_box = GTK_BOX(gtk_vbox_new(false, 0));
-	gtk_container_add(GTK_CONTAINER(m_window), GTK_WIDGET(m_window_box));
-
 	// Create the border of the window
-	m_window_contents = gtk_frame_new(NULL);
-	gtk_frame_set_shadow_type(GTK_FRAME(m_window_contents), GTK_SHADOW_OUT);
-	gtk_box_pack_start(m_window_box, m_window_contents, true, true, 0);
+	GtkWidget* frame = gtk_frame_new(NULL);
+	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
+	gtk_container_add(GTK_CONTAINER(m_window), frame);
 
 	// Create the profile picture
 	m_profilepic = new ProfilePicture(this);
@@ -110,7 +108,7 @@ WhiskerMenu::Window::Window() :
 
 	// Create search entry
 	m_search_entry = GTK_ENTRY(gtk_entry_new());
-	gtk_entry_set_icon_from_stock(m_search_entry, GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_FIND);
+	gtk_entry_set_icon_from_icon_name(m_search_entry, GTK_ENTRY_ICON_SECONDARY, "edit-find");
 	gtk_entry_set_icon_activatable(m_search_entry, GTK_ENTRY_ICON_SECONDARY, false);
 	g_signal_connect_slot<GtkEditable*>(m_search_entry, "changed", &Window::search, this);
 
@@ -147,7 +145,7 @@ WhiskerMenu::Window::Window() :
 
 	// Create box for packing children
 	m_vbox = GTK_BOX(gtk_vbox_new(false, 6));
-	gtk_container_add(GTK_CONTAINER(m_window_contents), GTK_WIDGET(m_vbox));
+	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(m_vbox));
 	gtk_container_set_border_width(GTK_CONTAINER(m_vbox), 2);
 
 	// Create box for packing commands
@@ -185,10 +183,10 @@ WhiskerMenu::Window::Window() :
 	gtk_box_pack_start(m_panels_box, m_applications->get_widget(), true, true, 0);
 
 	// Create box for packing sidebar
-	m_sidebar_box = GTK_BOX(gtk_vbox_new(false, 0));
-	gtk_box_pack_start(m_sidebar_box, GTK_WIDGET(m_favorites_button->get_button()), false, false, 0);
-	gtk_box_pack_start(m_sidebar_box, GTK_WIDGET(m_recent_button->get_button()), false, false, 0);
-	gtk_box_pack_start(m_sidebar_box, gtk_hseparator_new(), false, true, 0);
+	m_sidebar_buttons = GTK_BOX(gtk_vbox_new(false, 0));
+	gtk_box_pack_start(m_sidebar_buttons, GTK_WIDGET(m_favorites_button->get_button()), false, false, 0);
+	gtk_box_pack_start(m_sidebar_buttons, GTK_WIDGET(m_recent_button->get_button()), false, false, 0);
+	gtk_box_pack_start(m_sidebar_buttons, gtk_hseparator_new(), false, true, 0);
 
 	m_sidebar = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
 	gtk_box_pack_start(m_contents_box, GTK_WIDGET(m_sidebar), false, false, 0);
@@ -199,32 +197,31 @@ WhiskerMenu::Window::Window() :
 		gtk_scrolled_window_get_vadjustment(m_sidebar));
 	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_NONE);
 	gtk_container_add(GTK_CONTAINER(m_sidebar), viewport);
-	gtk_container_add(GTK_CONTAINER(viewport), GTK_WIDGET(m_sidebar_box));
-
-	GtkSizeGroup* sidebar_size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
-	gtk_size_group_add_widget(sidebar_size_group, GTK_WIDGET(m_sidebar));
-	gtk_size_group_add_widget(sidebar_size_group, GTK_WIDGET(m_commands_align));
+	gtk_container_add(GTK_CONTAINER(viewport), GTK_WIDGET(m_sidebar_buttons));
 
 	// Show widgets
-	gtk_widget_show_all(GTK_WIDGET(m_window_box));
+	gtk_widget_show_all(frame);
 	gtk_widget_hide(m_favorites->get_widget());
 	gtk_widget_hide(m_recent->get_widget());
 	gtk_widget_hide(m_applications->get_widget());
 	gtk_widget_hide(m_search_results->get_widget());
 	m_default_button->set_active(true);
-	gtk_widget_show(m_window_contents);
+	gtk_widget_show(frame);
 
 	// Resize to last known size
 	gtk_window_set_default_size(m_window, m_geometry.width, m_geometry.height);
 
 	// Handle transparency
-	gtk_widget_set_app_paintable(GTK_WIDGET(m_sidebar_box), true);
-	g_signal_connect_slot(m_sidebar_box, "expose-event", &Window::on_expose_event, this);
+	gtk_widget_set_app_paintable(GTK_WIDGET(m_sidebar_buttons), true);
+	g_signal_connect_slot(m_sidebar_buttons, "expose-event", &Window::on_expose_event, this);
 	gtk_widget_set_app_paintable(GTK_WIDGET(m_window), true);
 	g_signal_connect_slot(m_window, "expose-event", &Window::on_expose_event, this);
 	g_signal_connect_slot(m_window, "screen-changed", &Window::on_screen_changed_event, this);
 	on_screen_changed_event(GTK_WIDGET(m_window), NULL);
 
+	// Load applications
+	m_applications->load_applications();
+
 	g_object_ref_sink(m_window);
 }
 
@@ -278,15 +275,21 @@ void WhiskerMenu::Window::hide()
 
 void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
 {
-	// Make sure icon sizes are correct
-	m_favorites_button->reload_icon_size();
-	m_recent_button->reload_icon_size();
-	m_applications->reload_category_icon_size();
-
-	m_search_results->get_view()->reload_icon_size();
-	m_favorites->get_view()->reload_icon_size();
-	m_recent->get_view()->reload_icon_size();
-	m_applications->get_view()->reload_icon_size();
+	// Handle showing tooltips
+	if (wm_settings->launcher_show_tooltip)
+	{
+		m_search_results->get_view()->show_tooltips();
+		m_favorites->get_view()->show_tooltips();
+		m_recent->get_view()->show_tooltips();
+		m_applications->get_view()->show_tooltips();
+	}
+	else
+	{
+		m_search_results->get_view()->hide_tooltips();
+		m_favorites->get_view()->hide_tooltips();
+		m_recent->get_view()->hide_tooltips();
+		m_applications->get_view()->hide_tooltips();
+	}
 
 	// Make sure commands are valid and visible
 	for (int i = 0; i < Settings::CountCommands; ++i)
@@ -319,6 +322,16 @@ void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
 	}
 	show_default_page();
 
+	// Make sure icon sizes are correct
+	m_favorites_button->reload_icon_size();
+	m_recent_button->reload_icon_size();
+	m_applications->reload_category_icon_size();
+
+	m_search_results->get_view()->reload_icon_size();
+	m_favorites->get_view()->reload_icon_size();
+	m_recent->get_view()->reload_icon_size();
+	m_applications->get_view()->reload_icon_size();
+
 	GdkScreen* screen = NULL;
 	int parent_x = 0, parent_y = 0, parent_w = 0, parent_h = 0;
 	if (parent != NULL)
@@ -424,20 +437,29 @@ void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
 	{
 		m_layout_left = !layout_left;
 		m_layout_commands_alternate = wm_settings->position_commands_alternate;
+
+		g_object_ref(m_commands_align);
 		if (m_layout_commands_alternate)
 		{
-			g_object_ref(m_commands_align);
+			if (!wm_settings->position_categories_alternate)
+			{
+				m_sidebar_size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+				gtk_size_group_add_widget(m_sidebar_size_group, GTK_WIDGET(m_sidebar));
+				gtk_size_group_add_widget(m_sidebar_size_group, GTK_WIDGET(m_commands_align));
+			}
+
 			gtk_container_remove(GTK_CONTAINER(m_title_box), GTK_WIDGET(m_commands_align));
 			gtk_box_pack_start(m_search_box, GTK_WIDGET(m_commands_align), false, false, 0);
-			g_object_unref(m_commands_align);
 		}
 		else
 		{
-			g_object_ref(m_commands_align);
+			g_object_unref(m_sidebar_size_group);
+			m_sidebar_size_group = NULL;
+
 			gtk_container_remove(GTK_CONTAINER(m_search_box), GTK_WIDGET(m_commands_align));
 			gtk_box_pack_start(m_title_box, GTK_WIDGET(m_commands_align), false, false, 0);
-			g_object_unref(m_commands_align);
 		}
+		g_object_unref(m_commands_align);
 	}
 	if ((layout_left && !wm_settings->position_categories_alternate)
 			|| (!layout_left && wm_settings->position_categories_alternate))
@@ -549,6 +571,9 @@ void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
 		}
 	}
 
+	// Make sure recent button is only visible when tracked
+	gtk_widget_set_visible(GTK_WIDGET(m_recent_button->get_button()), wm_settings->recent_items_max);
+
 	// Show window
 	gtk_widget_show(GTK_WIDGET(m_window));
 	gtk_window_move(m_window, m_geometry.x, m_geometry.y);
@@ -589,10 +614,10 @@ void WhiskerMenu::Window::set_categories(const std::vector<SectionButton*>& cate
 	for (std::vector<SectionButton*>::const_iterator i = categories.begin(), end = categories.end(); i != end; ++i)
 	{
 		(*i)->set_group(m_recent_button->get_group());
-		gtk_box_pack_start(m_sidebar_box, GTK_WIDGET((*i)->get_button()), false, false, 0);
+		gtk_box_pack_start(m_sidebar_buttons, GTK_WIDGET((*i)->get_button()), false, false, 0);
 		g_signal_connect_slot<GtkToggleButton*>((*i)->get_button(), "toggled", &Window::category_toggled, this);
 	}
-	gtk_widget_show_all(GTK_WIDGET(m_sidebar_box));
+	gtk_widget_show_all(GTK_WIDGET(m_sidebar_buttons));
 
 	show_default_page();
 }
@@ -691,28 +716,40 @@ gboolean WhiskerMenu::Window::on_key_press_event(GtkWidget* widget, GdkEvent* ev
 		return true;
 	}
 
-	// Make up and down keys always scroll current list of applications
+	Page* page = NULL;
+	if (gtk_widget_get_visible(m_search_results->get_widget()))
+	{
+		page = m_search_results;
+	}
+	else if (m_favorites_button->get_active())
+	{
+		page = m_favorites;
+	}
+	else if (m_recent_button->get_active())
+	{
+		page = m_recent;
+	}
+	else
+	{
+		page = m_applications;
+	}
+	GtkWidget* view = page->get_view()->get_widget();
+
+	// Allow keyboard navigation out of treeview
+	if ((key_event->keyval == GDK_KEY_Left) || (key_event->keyval == GDK_KEY_Right))
+	{
+		if ((widget == view) || (gtk_window_get_focus(m_window) == view))
+		{
+			gtk_widget_grab_focus(GTK_WIDGET(m_favorites_button->get_button()));
+			page->reset_selection();
+		}
+	}
+
+	// Make up and down keys scroll current list of applications from search
 	if ((key_event->keyval == GDK_KEY_Up) || (key_event->keyval == GDK_KEY_Down))
 	{
-		GtkWidget* view = NULL;
-		if (gtk_widget_get_visible(m_search_results->get_widget()))
-		{
-			view = m_search_results->get_view()->get_widget();
-		}
-		else if (m_favorites_button->get_active())
-		{
-			view = m_favorites->get_view()->get_widget();
-		}
-		else if (m_recent_button->get_active())
-		{
-			view = m_recent->get_view()->get_widget();
-		}
-		else
-		{
-			view = m_applications->get_view()->get_widget();
-		}
-
-		if ((widget != view) && (gtk_window_get_focus(m_window) != view))
+		GtkWidget* search = GTK_WIDGET(m_search_entry);
+		if ((widget == search) || (gtk_window_get_focus(m_window) == search))
 		{
 			gtk_widget_grab_focus(view);
 		}
@@ -898,7 +935,7 @@ void WhiskerMenu::Window::search()
 
 	// Update search entry icon
 	bool visible = text != NULL;
-	gtk_entry_set_icon_from_stock(m_search_entry, GTK_ENTRY_ICON_SECONDARY, !visible ? GTK_STOCK_FIND : GTK_STOCK_CLEAR);
+	gtk_entry_set_icon_from_icon_name(m_search_entry, GTK_ENTRY_ICON_SECONDARY, !visible ? "edit-find" : "edit-clear");
 	gtk_entry_set_icon_activatable(m_search_entry, GTK_ENTRY_ICON_SECONDARY, visible);
 
 	if (visible)
